/**************************************************
Mex-C code for reproducing experiment 2: find and sketch over multiple scales
***************************************************/

# include <stdio.h>
# include <stdlib.h>
# include "mex.h"        /* the algorithm is connect to matlab */
# include "math.h"
# define PI 3.1415926
# define ABS(x) ((x)>0? (x):(-(x)))
# define MAX(x, y) ((x)>(y)? (x):(y))
# define MIN(x, y) ((x)<(y)? (x):(y))
# define NEGMAX -1e10

double *double_vector(int n)
{
    double *v; 
    
    v = (double*) mxCalloc (n, sizeof(double));
    
    return v; 
}

int *int_vector(int n)
{
    int *v; 
    
    v = (int*) mxCalloc (n, sizeof(int));
    
    return v; 
}

double **double_matrix(int m, int n)
{
    double **mat; 
    int i; 
    
    mat = (double**) mxCalloc(m, sizeof(double*)); 
    for (i=0; i<m; i++)
        mat[i] = double_vector(n); 
    
    return mat; 
}

int **int_matrix(int m, int n)
{
    int **mat; 
    int i; 
    
    mat = (int**) mxCalloc(m, sizeof(int*)); 
    for (i=0; i<m; i++)
        mat[i] = int_vector(n); 
    
    return mat; 
}

void free_matrix(void **a, int nrow, int ncol)
{
	int i;
	for(i = 0; i < nrow; i++)
		mxFree(a[i]);
	mxFree(a);
}


/* getting the index of matlab image, (x,y) location, (sx, sy) sizes */
int px(int x, int y, int bx, int by)    
{            
   return (x + (y-1)*bx - 1); 
 }

 /* variables */

 int sx, sy;                 /* current size of image */
 int sub;                    /* subsampling */
 double *tss;                /* map of log-likelihood score */
 double *thatx, *thaty, *M2All; 
 int di; 

 
 
/* the shared sketch algorithm */
void Cmp()
{
   int x0, y0, x, y, here; 
   double m; 
   
    M2All[di-1] = NEGMAX; 
    for (x=1; x<=floor(sx/sub); x++)
         for (y=1; y<=floor(sy/sub); y++)
          {            
           x0 = x*sub; y0 = y*sub;   
           here = px(x0, y0, sx, sy);    
           if (M2All[di-1]<tss[here])
           {
              M2All[di-1] = tss[here]; 
              thatx[di-1] = x0; 
              thaty[di-1] = y0;
           }
          }   
}


/* mex function is used to pass on the pointers and scalars from matlab, 
   so that heavy computation can be done by C, which puts the results into 
   some of the pointers. After that, matlab can then use these results. 
   
   So matlab is very much like a managing platform for organizing the 
   experiments, and mex C is like a work enginee for fast computation. */

void mexFunction(int nlhs, mxArray *plhs[], 
                 int nrhs, const mxArray *prhs[])                
{
 int ind, c; 
 mxArray *f;  
 
 c = 0; /* counter for input pointers and scalars */
 di = floor(mxGetScalar(prhs[c++])+.5);
 sub = floor(mxGetScalar(prhs[c++])+.5);   /* sub-sampling of Gabor filters to improve speed */
 tss = mxGetPr(prhs[c++]); 
 sx = floor(mxGetScalar(prhs[c++])+.5); 
 sy = floor(mxGetScalar(prhs[c++])+.5); 
 thatx = mxGetPr(prhs[c++]); 
 thaty = mxGetPr(prhs[c++]); 
 M2All = mxGetPr(prhs[c++]); 
 
 Cmp();     
}

     



 

                    